#include "FMODUtility.hpp"
#include "Sound3DObject.hpp"

void FMODUtil::ERRCHECK(FMOD_RESULT result)
{
    if (result != FMOD_OK)
    {
		OutputDebugStr(FMOD_ErrorString(result));
		exit(-1);
    }
}

//Init FMOD with the number of sound channels wed like to use.
int FMODManager::InitFMOD(int NumChannels)
{
	/*
        Create a System object and initialize.
    */
    result = FMOD::System_Create(&system);	// Create the FMod System (access to FMod functions), Pass a pointer to this function to store the system
    FMODUtil::ERRCHECK(result);

    result = system->getVersion(&version);	// Getting their current version of FMod
    FMODUtil::ERRCHECK(result);

    if (version < FMOD_VERSION)				// Check to make sure they have a compatible version of FMod
    {
        printf("Error!  You are using an old version of FMOD %08x.  This program requires %08x\n", version, FMOD_VERSION);
        return 0;
    }

	// Initialize our FMod system
	// Parameters: Max channels, flags, ??? (extra driver information)
    result = system->init(NumChannels, FMOD_INIT_NORMAL, 0);
    FMODUtil::ERRCHECK(result);

	listenerPos.x = 0.0f;
	listenerPos.y = 0.0f;
	listenerPos.z = 0.0f;

	listenerVel.x = 0.0f;
	listenerVel.y = 0.0f;
	listenerVel.z = 0.0f;

	forward.x = 0.0f;
	forward.y = 0.0f;
	forward.z = 1.0f;

	up.x = 0.0f;
	up.y = 1.0f;
	up.z = 0.0f;

	system->set3DListenerAttributes(0,  &listenerPos,  &listenerVel, &forward, &up);
	return 1;
}

//Create a sound based on a path, pass in a reference to a pointer for our sound, flag for if it's streaming, creation flags.
void FMODManager::CreateSound(const char *soundPath, FMOD::Sound *&sound,  bool isStream, FMOD_MODE createFlags)
{
	/*if(isStream)
	{
		//Create a sound that is streamed from the users harddrive.
		result = system->createStream(soundPath, createFlags, 0, &sound);
	}
	else
	{*/
		//Create a sound that is stored in memory.
		result = system->createSound(soundPath, FMOD_3D, 0, &sound);
	//}
	
	FMODUtil::ERRCHECK(result);
}

//Lets pass the sound we want to play, the channel well be using to play it, and if we want to grab a new channel or reuse it.
void FMODManager::PlayFSound(FMOD::Sound *sound, FMOD::Channel *&channel, bool reuseChannel)
{
	if (!reuseChannel)
	{
		//If the current channel is used, just grab any free channel.
		result = system->playSound(FMOD_CHANNEL_FREE, sound, false, &channel);
	}
	else
	{
		//Make sure we use this channel, get rid of any other sound playing on it.
		result = system->playSound(FMOD_CHANNEL_REUSE, sound, false, &channel);
	}

	FMODUtil::ERRCHECK(result);
}

//Stop all of our channels.
void FMODManager::StopAllChannels()
{
	//Our mastergroup is a group containing all of our channels. Any effects applied to it will apply across all channels.
	FMOD::ChannelGroup* masterGroup = NULL;
    system->getMasterChannelGroup(&masterGroup); 
 
	masterGroup->stop();
}

//Pause all of our channels.
void FMODManager::PauseAllChannels()
{
	//Our mastergroup is a group containing all of our channels. Any effects applied to it will apply across all channels.
	FMOD::ChannelGroup* masterGroup = NULL;
    system->getMasterChannelGroup(&masterGroup); 
    if (masterGroup) 
    { 
        bool paused; 
        masterGroup->getPaused(&paused); //Store our pause state in paused.
        masterGroup->setPaused(!paused); //Flip the pause bool.
    } 
}

//Pause a single channel.
void FMODManager::PauseChannelToggle(FMOD::Channel *&channel)
{
	bool paused;
	channel->getPaused(&paused); //Store our pause state in paused.
	channel->setPaused(!paused); //Flip the pause bool.
}

//Pause a single channel.
void FMODManager::UnpauseChannel(FMOD::Channel *&channel)
{
	channel->setPaused(false); //Flip the pause bool.
}

//Pause a single channel.
void FMODManager::PauseChannel(FMOD::Channel *&channel)
{
	channel->setPaused(true); //Flip the pause bool.
}

void FMODManager::CreateDSP(FMOD_DSP_TYPE dspType, FMOD::DSP *&curDSP)
{
	result = system->createDSPByType(dspType, &curDSP);
	FMODUtil::ERRCHECK(result);
}

void FMODManager::SetDSP(FMOD::DSP *&curDSP)
{
    bool active;

    result = curDSP->getActive(&active);
	FMODUtil::ERRCHECK(result);

    if (!active)
    {
        result = system->addDSP(curDSP, 0);
		FMODUtil::ERRCHECK(result);
    }
}

void FMODManager::RemoveDSP(FMOD::DSP *&curDSP)
{
	bool active;

    result = curDSP->getActive(&active);
	FMODUtil::ERRCHECK(result);

    if (active)
    {
        result = curDSP->remove();
		FMODUtil::ERRCHECK(result);
    }
}

void FMODManager::ToggleDSP(FMOD::DSP *&curDSP)
{
	bool active;

    result = curDSP->getActive(&active);
	FMODUtil::ERRCHECK(result);

	if (active)
	{
		RemoveDSP(curDSP);
	}
	else
	{
		SetDSP(curDSP);
	}
}

void FMODManager::SetListenerPosition(float xPos, float yPos, float zPos)
{
	listenerPos.x = xPos;
	listenerPos.y = yPos;
	listenerPos.z = zPos;

	listenerVel.x = 0;
	listenerVel.y = 0;
	listenerVel.z = 0;

	FMOD_VECTOR forward;
	forward.x = 0;
	forward.y = 0;
	forward.z = 1;

	FMOD_VECTOR up;
	forward.x = 0;
	forward.y = 1;
	forward.z = 0;

	system->set3DListenerAttributes(0,  &listenerPos,  &listenerVel, &forward, &up);
}